@@ -1,4 +1,4 @@
-/*	$NetBSD: vbus.c,v 1.2 2016/06/21 20:36:02 palle Exp $	*/
+/*	$NetBSD: vbus.c,v 1.4 2019/11/10 21:16:33 chs Exp $	*/
 /*	$OpenBSD: vbus.c,v 1.8 2015/09/27 11:29:20 kettenis Exp $	*/
 /*
  * Copyright (c) 2008 Mark Kettenis
@@ -155,19 +155,17 @@ vbus_intr_map(int node, int ino, uint64_t *sysino)
 	len = OF_getproplen(parent, "interrupt-map-mask");
 	if (len < (address_cells + interrupt_cells) * sizeof(int))
 		return (-1);
-	imap_mask = malloc(len, M_DEVBUF, M_NOWAIT);
-	if (imap_mask == NULL)
-		return (-1);
+	imap_mask = malloc(len, M_DEVBUF, M_WAITOK);
 	if (OF_getprop(parent, "interrupt-map-mask", imap_mask, len) != len)
-		return (-1);
+		goto out;
 
 	if (prom_getprop(parent, "interrupt-map", sizeof(int), &nimap, (void **)&imap))
-	  panic("vbus: can't get interrupt-map");
+		panic("vbus: can't get interrupt-map");
 
 	if (prom_getprop(node, "reg", sizeof(*reg), &nreg, (void **)&reg))
-		  panic("vbus: can't get reg");
+		panic("vbus: can't get reg");
 	if (nreg < address_cells)
-		return (-1);
+		goto out;
 
 	while (nimap >= address_cells + interrupt_cells + 2) {
 		if (vbus_cmp_cells(imap, reg, imap_mask, address_cells) &&
@@ -180,21 +178,24 @@ vbus_intr_map(int node, int ino, uint64_t *sysino)
 			reg = NULL;
 
 			if (prom_getprop(node, "reg", sizeof(*reg), &nreg, (void **)&reg))
-			  panic("vbus: can't get reg");
+				panic("vbus: can't get reg");
 
 			devhandle = reg[0] & 0x0fffffff;
 
 			err = hv_intr_devino_to_sysino(devhandle, devino, sysino);
 			if (err != H_EOK)
-			  return (-1);
+				goto out;
 
 			KASSERT(*sysino == INTVEC(*sysino));
+			free(imap_mask, M_DEVBUF);
 			return (0);
 		}
 		imap += address_cells + interrupt_cells + 2;
 		nimap -= address_cells + interrupt_cells + 2;
 	}
 
+out:
+	free(imap_mask, M_DEVBUF);
 	return (-1);
 }
 
@@ -260,10 +261,7 @@ vbus_alloc_bus_tag(struct vbus_softc *sc, bus_space_tag_t parent)
 {
 	struct sparc_bus_space_tag *bt;
 
-	bt = malloc(sizeof(*bt), M_DEVBUF, M_NOWAIT | M_ZERO);
-	if (bt == NULL)
-		panic("could not allocate vbus bus tag");
-
+	bt = malloc(sizeof(*bt), M_DEVBUF, M_WAITOK | M_ZERO);
 	bt->cookie = sc;
 	bt->parent = parent;
 	bt->sparc_bus_map = parent->sparc_bus_map;
