@@ -1,4 +1,4 @@
-/*	$NetBSD: vofdisk.c,v 1.51 2015/04/26 15:15:20 mlelstv Exp $	*/
+/*	$NetBSD: ofdisk.c,v 1.54 2020/01/26 21:43:52 thorpej Exp $	*/
 
 /*
  * Copyright (C) 1995, 1996 Wolfgang Solfrank.
@@ -32,7 +32,7 @@
  */
 
 #include <sys/cdefs.h>
-__KERNEL_RCSID(0, "$NetBSD: vofdisk.c,v 1.51 2015/04/26 15:15:20 mlelstv Exp $");
+__KERNEL_RCSID(0, "$NetBSD: ofdisk.c,v 1.54 2020/01/26 21:43:52 thorpej Exp $");
 
 #include <sys/param.h>
 #include <sys/buf.h>
@@ -47,6 +47,7 @@ __KERNEL_RCSID(0, "$NetBSD: vofdisk.c,v 1.51 2015/04/26 15:15:20 mlelstv Exp $")
 #include <sys/proc.h>
 
 #include <dev/ofw/openfirm.h>
+
 #include <sparc64/dev/vbusvar.h>
 
 struct vofdisk_softc {
@@ -120,38 +121,29 @@ void vofdisk_getdisklabel (dev_t);
 static int
 vofdisk_match(device_t parent, cfdata_t match, void *aux)
 {
-#if 1
-/* FIXME */
-return 0;
-#endif
 	struct vbus_attach_args *va = aux;
-	char type[8];
-	int l;
 
-#if 0
-FIXME leftover from ofdisk
-	if (strcmp(oba->oba_busname, "ofw"))
-		return (0);
-#endif
-	if ((l = OF_getprop(va->va_node, "device_type", type,
-	    sizeof type - 1)) < 0)
-		return 0;
-	if (l >= sizeof type)
-		return 0;
-	type[l] = 0;
-	return !strcmp(type, "block");
+	if (strcmp(va->va_name, "disk") == 0)
+		return (1);
+	/* FIXME more checks to be done
+	   type        block
+	   compatible  SUNW,legion-disk
+	*/
+
+	return (0);
 }
 
 static void
 vofdisk_attach(device_t parent, device_t self, void *aux)
 {
-	struct vofdisk_softc *of = device_private(self);
-	struct vbus_attach_args *va = aux;
+	struct vofdisk_softc *vofd = device_private(self);
+#if 0	
+	struct ofbus_attach_args *oba = aux;
 	char child[64];
 	int l;
 
 	of->sc_dev = self;
-	if ((l = OF_getprop(va->va_node, "name", child,
+	if ((l = OF_getprop(oba->oba_phandle, "name", child,
 	    sizeof child - 1)) < 0)
 		panic("device without name?");
 	if (l >= sizeof child)
@@ -159,25 +151,21 @@ vofdisk_attach(device_t parent, device_t self, void *aux)
 	child[l] = 0;
 
 	of->sc_flags = 0;
-	of->sc_phandle = va->va_node;
-printf("vofdisk_attach() sc_phandle [%d]\n", of->sc_phandle);
-#if 0
-ofdisk leftover
+	of->sc_phandle = oba->oba_phandle;
 	of->sc_unit = oba->oba_unit;
-#else
-	of->sc_unit = 0;
-#endif
 	of->sc_ihandle = 0;
-	disk_init(&of->sc_dk, device_xname(of->sc_dev), &vofdisk_dkdriver);
-	disk_attach(&of->sc_dk);
+#endif	
+	disk_init(&vofd->sc_dk, device_xname(/*vofd->sc_dev*/self), &vofdisk_dkdriver);
+	disk_attach(&vofd->sc_dk);
 	printf("\n");
-
+#if 0
 	if (strcmp(child, "floppy") == 0)
-		of->sc_flags |= OFDF_ISFLOPPY;
+		vofd->sc_flags |= VOFDF_ISFLOPPY;
 	else {
 		/* Discover wedges on this disk. */
-		dkwedge_discover(&of->sc_dk);
+		dkwedge_discover(&vofd->sc_dk);
 	}
+#endif	
 }
 
 int
@@ -212,7 +200,6 @@ vofdisk_open(dev_t dev, int flags, int fmt, struct lwp *lwp)
 			goto bad1;
 		}
 		path[l] = 0;
-printf("vofdisk_open() path [%s]\n", path);
 
 		/*
 		 * XXX This is for the benefit of SCSI/IDE disks that don't
@@ -222,16 +209,12 @@ printf("vofdisk_open() path [%s]\n", path);
 		 * XXX See also ofscsi.c
 		 */
 		if (!strcmp(path + l - 4, "disk")) {
-printf("vofdisk_open() hack path [%s]\n", path);
 			path[l++] = '@';
 			path[l++] = '0' + of->sc_unit;
 			path[l] = 0;
-printf("vofdisk_open() hack path [%s]\n", path);
 		}
 
 		strlcat(path, ":0", sizeof(path));
-printf("vofdisk_open() path before OF_open() [%s]\n", path);
-printf("vofdisk_open() &path %p\n", (void*)&path);
 
 		if ((of->sc_ihandle = OF_open(path)) == -1) {
 			error = ENXIO;
@@ -386,6 +369,7 @@ vofdisk_ioctl(dev_t dev, u_long cmd, void *data, int flag, struct lwp *l)
 	case DIOCDWEDGE:
 	case DIOCAWEDGE:
 	case DIOCLWEDGES:
+	case DIOCRMWEDGES:
 	case DIOCMWEDGES:
 		if (VOFDISK_FLOPPY_P(of))
 			return ENOTTY;
@@ -422,11 +406,11 @@ vofdisk_ioctl(dev_t dev, u_long cmd, void *data, int flag, struct lwp *l)
 		error = setdisklabel(of->sc_dk.dk_label,
 		    lp, /*of->sc_dk.dk_openmask */0,
 		    of->sc_dk.dk_cpulabel);
-		if (error == 0 && cmd == DIOCWDINFO
+		if (error == 0 && (cmd == DIOCWDINFO
 #ifdef __HAVE_OLD_DISKLABEL
-		    || xfer == ODIOCWDINFO
+		    || cmd == ODIOCWDINFO
 #endif
-		    )
+		    ))
 			error = writedisklabel(MAKEDISKDEV(major(dev),
 			    DISKUNIT(dev), RAW_PART), vofdisk_strategy,
 			    of->sc_dk.dk_label, of->sc_dk.dk_cpulabel);
@@ -440,7 +424,7 @@ vofdisk_ioctl(dev_t dev, u_long cmd, void *data, int flag, struct lwp *l)
 		vofdisk_getdefaultlabel(of, (struct disklabel *)data);
 		return 0;
 #ifdef __HAVE_OLD_DISKLABEL
-	case DIOCGDEFLABEL:
+	case ODIOCGDEFLABEL:
 		vofdisk_getdefaultlabel(of, &newlabel);
 		if (newlabel.d_npartitions > OLDMAXPARTITIONS)
 			return ENOTTY;
