@@ -1,4 +1,4 @@
-/*	$NetBSD: trap.c,v 1.191 2019/02/15 16:36:33 hannken Exp $ */
+/*	$NetBSD: trap.c,v 1.192 2019/11/21 19:24:01 ad Exp $ */
 
 /*
  * Copyright (c) 1996-2002 Eduardo Horvath.  All rights reserved.
@@ -50,7 +50,7 @@
  */
 
 #include <sys/cdefs.h>
-__KERNEL_RCSID(0, "$NetBSD: trap.c,v 1.191 2019/02/15 16:36:33 hannken Exp $");
+__KERNEL_RCSID(0, "$NetBSD: trap.c,v 1.192 2019/11/21 19:24:01 ad Exp $");
 
 #include "opt_ddb.h"
 #include "opt_multiprocessor.h"
@@ -124,7 +124,7 @@ int rftkcnt[5] = { 0, 0, 0, 0, 0 };
 #define RW_64		0x1
 #define RW_ERR		0x2
 #define RW_FOLLOW	0x4
-int	rwindow_debug = RW_ERR|RW_64|RW_FOLLOW;
+int	rwindow_debug = RW_ERR;
 #define TDB_ADDFLT	0x1
 #define TDB_TXTFLT	0x2
 #define TDB_TRAP	0x4
@@ -137,7 +137,7 @@ int	rwindow_debug = RW_ERR|RW_64|RW_FOLLOW;
 #define TDB_STOPCALL	0x200
 #define TDB_STOPCPIO	0x400
 #define TDB_SYSTOP	0x800
-int	trapdebug = 0/*TDB_FOLLOW|TDB_TRAP|TDB_TL*//*|TDB_SYSCALL|TDB_STOPSIG|TDB_STOPCPIO|TDB_ADDFLT|TDB_FOLLOW*/;
+int	trapdebug = 0/*|TDB_SYSCALL|TDB_STOPSIG|TDB_STOPCPIO|TDB_ADDFLT|TDB_FOLLOW*/;
 /* #define inline */
 #endif
 
@@ -370,7 +370,7 @@ void text_access_error(struct trapframe64 *, unsigned int, vaddr_t, u_long,
 	vaddr_t, u_long);
 void ecc_corrected_error(unsigned int type, vaddr_t pc);
 
-#if 1
+#ifdef DEBUG
 void print_trapframe(struct trapframe64 *);
 
 void
@@ -393,7 +393,7 @@ print_trapframe(struct trapframe64 *tf)
 	       (u_int)(tf->tf_global[5]>>32), (u_int)tf->tf_global[5],
 	       (u_int)(tf->tf_global[6]>>32), (u_int)tf->tf_global[6],
 	       (u_int)(tf->tf_global[7]>>32), (u_int)tf->tf_global[7]);
-#if 1
+#ifdef DEBUG
 	printf("%08x%08x %08x%08x %08x%08x %08x%08x\n",
 	       (u_int)(tf->tf_out[0]>>32), (u_int)tf->tf_out[0],
 	       (u_int)(tf->tf_out[1]>>32), (u_int)tf->tf_out[1],
@@ -456,6 +456,7 @@ trap(struct trapframe64 *tf, unsigned int type, vaddr_t pc, long tstate)
 
 	/* This steps the PC over the trap. */
 #define	ADVANCE (n = tf->tf_npc, tf->tf_pc = n, tf->tf_npc = n + 4)
+
 #ifdef DEBUG
 	if (tf->tf_pc == tf->tf_npc) {
 		printf("trap: tpc %p == tnpc %p\n",
@@ -612,39 +613,10 @@ dopanic:
 			l->l_pflag &= ~LP_OWEUPC;
 			ADDUPROF(l);
 		}
-		if (want_resched)
-			preempt();
 		break;
 
-#if 0
-	case T_RWRET:
-		/*
-		 * XXX Flushing the user windows here should not be
-		 * necessary, but not doing so here causes corruption
-		 * of user windows on sun4v.  Flushing them shouldn't
-		 * be much of a performance penalty since we're
-		 * probably going to spill any remaining user windows
-		 * anyhow.
-		 */
-		write_user_windows();
-#if 1		
-		if (rwindow_save(l) == -1) {
-#if 0		  
-			KERNEL_LOCK();
-			trapsignal(p, SIGILL, 0, ILL_BADSTK, sv);
-			KERNEL_UNLOCK();
-#else
-			printf("rwindow_save() failed\n");
-			Debugger();
-#endif			  
-		}
-#endif		
-		break;
-#endif
-
 	case T_INST_EXCEPT:
 	case T_TEXTFAULT:
-
 #ifdef DEBUG
 		/* This is not an MMU issue!!!! */
 		printf("trap: pid=%d.%d comm=%s textfault at %lx!! sending SIGILL due to trap %d: %s\n", 
@@ -759,21 +731,6 @@ dopanic:
 			tf->tf_npc = tf->tf_pc + 4;
 			break;
 		}
-
-		printf("trap align curpcb->pcb_nsaved %d\n", curpcb->pcb_nsaved);
-		printf("trap align pcb->pcb_nsaved %d\n", pcb->pcb_nsaved);
-		struct rwindow64 *rw = &pcb->pcb_rw[0];
-		for (int iii = 0; iii < PCB_MAXWIN; iii++) {
-		  printf("offset %d\n", iii);
-		  for (int ii = 0; ii < 8; ii++) {
-		    printf("%%l%d %016lx\n", ii, (long unsigned int)rw[iii].rw_local[ii]);
-		  }
-		  printf("\n");
-		  for (int ii = 0; ii < 8; ii++) {
-		    printf("%%i%d %016lx\n", ii, (long unsigned int)rw[iii].rw_in[ii]);
-		  }
-		  printf("\n");
-		}
 		
 #ifdef DEBUG
 #define fmt64(x)	(u_int)((x)>>32), (u_int)((x))
@@ -783,16 +740,50 @@ dopanic:
 			       l->l_proc->p_pid, l->l_lid, l->l_proc->p_comm, fmt64(dsfsr), fmt64(dsfar),
 			       fmt64(isfsr), pc);
 		} else {
-		  /* FIXME are the returned mmufsa values correct? */
+		  
+			printf("Alignment error: pid=%d.%d comm=%s pc=%lx\n",
+			       l->l_proc->p_pid, l->l_lid, l->l_proc->p_comm, pc);
+			paddr_t mmufsa_ift_addr = cpus->ci_mmufsa + offsetof(struct mmufsa, ift);
+			paddr_t mmufsa_ifa_addr = cpus->ci_mmufsa + offsetof(struct mmufsa, ifa);
+			paddr_t mmufsa_ifc_addr = cpus->ci_mmufsa + offsetof(struct mmufsa, ifc);
+			paddr_t mmufsa_dft_addr = cpus->ci_mmufsa + offsetof(struct mmufsa, dft);
 			paddr_t mmufsa_dfa_addr = cpus->ci_mmufsa + offsetof(struct mmufsa, dfa);
 			paddr_t mmufsa_dfc_addr = cpus->ci_mmufsa + offsetof(struct mmufsa, dfc);
+			int64_t ift = ldxa(mmufsa_ift_addr, ASI_PHYS_CACHED);
+			printf("ift = %016lx\n", ift);
+			int64_t ifa = ldxa(mmufsa_ifa_addr, ASI_PHYS_CACHED);
+			printf("ifa = %016lx\n", ifa);
+			int64_t ifc = ldxa(mmufsa_ifc_addr, ASI_PHYS_CACHED);
+			printf("ifc = %016lx\n", ifc);
+			int64_t dft = ldxa(mmufsa_dft_addr, ASI_PHYS_CACHED);
+			printf("dft = %016lx\n", dft);
 			int64_t dfa = ldxa(mmufsa_dfa_addr, ASI_PHYS_CACHED);
+			printf("dfa = %016lx\n", dfa);
 			int64_t dfc = ldxa(mmufsa_dfc_addr, ASI_PHYS_CACHED);
-			printf("Alignment error: pid=%d.%d comm=%s "
-			       "pc=%lx addr=%lx ctx=%ld\n",
-			       l->l_proc->p_pid, l->l_lid, l->l_proc->p_comm,
-			       pc, dfa, dfc);
-			Debugger();
+			printf("dfc = %016lx\n", dfc);
+			print_trapframe(tf);
+			printf("saved register windows %d\n", curpcb->pcb_nsaved);
+#if 1
+			if (curpcb->pcb_nsaved) {
+			  for (int iii = 0; iii < PCB_MAXWIN; iii++) {
+			    printf("saved register windows (%d) offset %d", curpcb->pcb_nsaved, iii);
+			    printf("\nlocals:");
+			    for (int iiii = 0; iiii < 8; iiii++) {
+			      if ( iiii % 4 == 0)
+				printf("\n");
+			      printf("%016lx ", curpcb->pcb_rw[iii].rw_local[iiii]);
+			    }
+			    printf("\nins:");
+			    for (int iiii = 0; iiii < 8; iiii++) {
+			      if ( iiii % 4 == 0)
+				printf("\n");
+			      printf("%016lx ", curpcb->pcb_rw[iii].rw_in[iiii]);
+			    }
+			    printf("\n");
+			  }
+			  printf("\n");
+			}
+#endif	
 		}
 #endif
 		
@@ -936,8 +927,9 @@ dopanic:
 		ksi.ksi_signo = sig;
 		trapsignal(l, &ksi);
 	}
-  
-	printf("trap(%x) calling useret()\n", type);
+#if 0
+	printf("trap(%x) before userret()\n", type);
+#endif	
 	userret(l, pc, sticks);
 	share_fpu(l, tf);
 #undef ADVANCE
@@ -949,8 +941,6 @@ dopanic:
 #endif
 }
 
-#if 1
-/* NetBSD */
 /*
  * Save windows from PCB into user stack, and return 0.  This is used on
  * window overflow pseudo-traps (from locore.s, just before returning to
@@ -965,12 +955,6 @@ dopanic:
 int
 rwindow_save(struct lwp *l)
 {
-  printf("rwindow_save()\n");
-#ifdef DEBUG
-#if 1
-  Debugger();
-#endif  
-#endif  
 	struct pcb *pcb = lwp_getpcb(l);
 	struct rwindow64 *rw = &pcb->pcb_rw[0];
 	uint64_t rwdest;
@@ -988,33 +972,8 @@ rwindow_save(struct lwp *l)
 		printf("%s[%d]: rwindow: pcb->stack:", l->l_proc->p_comm, l->l_proc->p_pid);
 #endif
 
-
-	
 	 while (i > 0) {
-	   
-#if 0	     
 		rwdest = rw[i--].rw_in[6];
-#else
-	printf("\n");
-	printf("\n");
-	printf("\n");
-	for (int iii = 0; iii < PCB_MAXWIN; iii++) {
-	  printf("offset %d\n", iii);
-	  for (int ii = 0; ii < 8; ii++) {
-	    printf("%%l%d %016lx\n", ii, (long unsigned int)rw[iii].rw_local[ii]);
-	  }
-	  printf("\n");
-	  for (int ii = 0; ii < 8; ii++) {
-	    printf("%%i%d %016lx\n", ii, (long unsigned int)rw[iii].rw_in[ii]);
-	  }
-	  printf("\n");
-	}
-
-	printf("i=%d\n", i);
-		rwdest = rw[i-1].rw_in[6];
-		printf("rwdest = %lx\n", (long unsigned int)rwdest);
-		i--;
-#endif		
 #ifdef DEBUG
 		if (rwindow_debug & RW_FOLLOW)
 			printf("window %d at %lx\n", i, (long)rwdest);
@@ -1026,10 +985,7 @@ rwindow_save(struct lwp *l)
 				       "or %p\n", 
 				       (void *)(long)rwdest,
 				       (void *)(long)(rwdest+BIAS));
-#if 0
-FIXME				
 				Debugger();
-#endif 
 			}
 #endif
 			rwdest += BIAS;
@@ -1048,10 +1004,7 @@ FIXME
 				printf("Finished copyout(%p, %p, %lx)\n",
 					(void *)&rw[i], (void *)(long)rwdest,
                                 	sizeof(*rw));
-#if 0
-FIXME				
 				Debugger();
-#endif 
 			}
 #endif
 		} else {
@@ -1080,41 +1033,12 @@ FIXME
 #ifdef DEBUG
 	if (rwindow_debug & RW_FOLLOW) {
 		printf("\n");
-#if 0
-FIXME		
 		Debugger();
-#endif 
 	}
 #endif
 	return (0);
 }
-#else
-/* OpenBSD */
-/*
- * Save windows from PCB into user stack, and return 0.  This is used on
- * window overflow pseudo-traps (from locore.s, just before returning to
- * user mode) and when ptrace or sendsig needs a consistent state.
- * As a side effect, rwindow_save() always sets pcb_nsaved to 0.
- *
- * If the windows cannot be saved, pcb_nsaved is restored and we return -1.
- */
-int
-rwindow_save(struct proc *p)
-{
-	struct pcb *pcb = &p->p_addr->u_pcb;
-	int i;
 
-	for (i = 0; i < pcb->pcb_nsaved; i++) {
-		pcb->pcb_rw[i].rw_in[7] ^= pcb->pcb_wcookie;
-		if (copyout(&pcb->pcb_rw[i], (void *)(pcb->pcb_rwsp[i] + BIAS),
-		    sizeof(struct rwindow64)))
-			return (-1);
-	}
-
-	pcb->pcb_nsaved = 0;
-	return (0);
-}
-#endif
 /*
  * Kill user windows (before exec) by writing back to stack or pcb
  * and then erasing any pcb tracks.  Otherwise we might try to write
@@ -1127,7 +1051,6 @@ cpu_vmspace_exec(struct lwp *l, vaddr_t start, vaddr_t end)
 
 	write_user_windows();
 	pcb->pcb_nsaved = 0;
-	printf("cpu_vmspace_exec() setting nsaved to zero\n");
 }
 
 /*
@@ -1149,29 +1072,7 @@ data_access_fault(struct trapframe64 *tf, unsigned int type, vaddr_t pc,
 	vaddr_t onfault;
 	u_quad_t sticks;
 	ksiginfo_t ksi;
-#if 0
-	int nsaved = curpcb->pcb_nsaved;
-#endif	
-#if 0	
-	printf("%ld: data_access_fault(%p, %x, %p, %p, %lx, %lx) "
-	       "nsaved=%d\n",
-	       (long)(curproc?curproc->p_pid:-1), tf, type,
-	       (void *)pc, (void *)addr, 
-	       sfva, sfsr, nsaved);
-#endif	
-#if 0
-	for (int i=0; i<nsaved; i++) {
-	  printf("rw index %d\n", i);
-	  printf("locals:\n");
-	  for (int j=0; j < 8; j++) {
-	    printf("%%l%d  %lx\n", j, (long unsigned int)curpcb->pcb_rw[i].rw_local[j]);
-	  }
-	  printf("ins:\n");
-	  for (int j=0; j < 8; j++) {
-	    printf("%%i%d  %lx\n", j, (long unsigned int)curpcb->pcb_rw[i].rw_in[j]);
-	  }
-	}
-#endif	
+
 #ifdef DEBUG
 	if (tf->tf_pc == tf->tf_npc) {
 		printf("data_access_fault: tpc %lx == tnpc %lx\n", 
@@ -1277,6 +1178,13 @@ data_access_fault(struct trapframe64 *tf, unsigned int type, vaddr_t pc,
 	/* alas! must call the horrible vm code */
 	pcb->pcb_onfault = NULL;
 	rv = uvm_fault(&vm->vm_map, va, access_type);
+	if (rv != 0) {
+	  printf("trapsignal() %s:%d va %p  rv %d \n", __FILE__, __LINE__, (void*)va, rv);
+#ifdef DEBUG	  
+	  print_trapframe(tf);
+#endif	  
+	  Debugger();
+	}
 	pcb->pcb_onfault = (void *)onfault;
 
 #ifdef DEBUG
@@ -1372,33 +1280,12 @@ kfault:
 		}
 		ksi.ksi_trap = type;
 		ksi.ksi_addr = (void *)sfva;
-		printf("data_access_fault() rv %d  sig %d  addr %p pc %p before trapsignal()\n",
-		       rv, ksi.ksi_signo, ksi.ksi_addr, (void*)pc);
-		print_trapframe(tf);
-#if 1
-		for (int i=0; i<PCB_MAXWIN; i++) {
-		  printf("rw index %d\n", i);
-		  printf("locals:");
-		  for (int j=0; j < 8; j++) {
-		    printf("  %%l%d  %016lx", j, (long unsigned int)pcb->pcb_rw[i].rw_local[j]);
-		  }
-		  printf("\n");
-		  printf("ins   :");
-		  for (int j=0; j < 8; j++) {
-		    printf("  %%i%d  %016lx", j, (long unsigned int)pcb->pcb_rw[i].rw_in[j]);
-		  }
-		  printf("\n");
-	  }
-#endif
-#if 1
-		Debugger();
-#endif		
-		
 		trapsignal(l, &ksi);
 	}
 	if ((tstate & TSTATE_PRIV) == 0) {
-	  printf("data_access_fault(type=%x, addr=%p, pc=%p) calling useret()\n",
-		 type, (void*)addr, (void*)pc);
+#if 0
+	  printf("data_access_fault() before userret()\n");
+#endif	  
 		userret(l, pc, sticks);
 		share_fpu(l, tf);
 	}
@@ -1437,12 +1324,6 @@ data_access_error(struct trapframe64 *tf, unsigned int type, vaddr_t afva,
 	u_quad_t sticks;
 	ksiginfo_t ksi;
 
-		printf("%d data_access_error(%lx, %lx, %lx, %p)=%lx @ %p\n",
-		       curproc?curproc->p_pid:-1, 
-		       (long)type, (long)sfva, (long)afva, tf,
-		       (long)tf->tf_tstate, 
-		       (void *)(u_long)tf->tf_pc);
-	
 #ifdef DEBUG
 	if (tf->tf_pc == tf->tf_npc) {
 		printf("data_access_error: tpc %lx == tnpc %lx\n", 
@@ -1575,7 +1456,9 @@ data_access_error(struct trapframe64 *tf, unsigned int type, vaddr_t afva,
 	trapsignal(l, &ksi);
 out:
 	if ((tstate & TSTATE_PRIV) == 0) {
-	  printf("data_access_error() calling useret()\n");
+#if 1	  
+	printf("data_access_error() before userret()\n");
+#endif	
 		userret(l, pc, sticks);
 		share_fpu(l, tf);
 	}
@@ -1605,10 +1488,7 @@ text_access_fault(struct trapframe64 *tf, unsigned int type, vaddr_t pc,
 	vm_prot_t access_type;
 	u_quad_t sticks;
 	ksiginfo_t ksi;
-#if 1
-		printf("text_access_fault entry: tpc %p\n",
-		    (void *)(u_long)tf->tf_pc);
-#endif	
+
 #ifdef DEBUG
 	if (tf->tf_pc == tf->tf_npc) {
 		printf("text_access_fault: tpc %p == tnpc %p\n",
@@ -1686,7 +1566,9 @@ text_access_fault(struct trapframe64 *tf, unsigned int type, vaddr_t pc,
 		ksi.ksi_addr = (void *)pc;
 		trapsignal(l, &ksi);
 	}
-	printf("text_access_fault() calling useret()\n");
+#if 0
+	printf("text_access_fault() before userret()\n");
+#endif	
 	userret(l, pc, sticks);
 	share_fpu(l, tf);
 #ifdef DEBUG
@@ -1698,10 +1580,6 @@ text_access_fault(struct trapframe64 *tf, unsigned int type, vaddr_t pc,
 		print_trapframe(tf);
 	}
 #endif
-#if 0	
-		printf("text_access_fault exit: tpc %p\n",
-		    (void *)(u_long)tf->tf_pc);
-#endif		       
 }
 
 
@@ -1726,9 +1604,6 @@ text_access_error(struct trapframe64 *tf, unsigned int type, vaddr_t pc,
 	u_quad_t sticks;
 	ksiginfo_t ksi;
 	char buf[768];
-
-		printf("text_access_error: tpc %p == tnpc %p\n",
-		    (void *)(u_long)tf->tf_pc, (void *)(u_long)tf->tf_npc);
 	
 #ifdef DEBUG
 	if (tf->tf_pc == tf->tf_npc) {
@@ -1836,7 +1711,9 @@ text_access_error(struct trapframe64 *tf, unsigned int type, vaddr_t pc,
 	}
 out:
 	if ((tstate & TSTATE_PRIV) == 0) {
-	printf("text_access_error() calling useret()\n");
+#if 1
+	printf("text_access_error() before userret()\n");
+#endif	
 		userret(l, pc, sticks);
 		share_fpu(l, tf);
 	}
